@@ -1,63 +1,48 @@
 ## slice(SPC, ...)
 
-# works on a single set of depths + property at a time
-# include:
-# 'bottom' - bottom boundary is included in the z-slice test
-# 'top' - top boundary is included in the z-slice test
-get.slice <- function(d, top, bottom, z, include='top', strict=TRUE) {
-  # extract pieces
-  d.top <- d[[top]]
-  d.bottom <- d[[bottom]]
-  d.v <- d[['value']]
-  d.var.name <- unique(d[['variable']]) # this is repeated for each horizon
-
-  # determine the property at z-slice, based on boundary rule
-  if(include == 'bottom')
-    res <- d.v[which(z > d.top & z <= d.bottom)]
+# this function is run on the horizon data, once for each depth slice
+get.slice <- function(h, id, top, bottom, vars, z, include='top', strict=TRUE) {
+  
+  # 1. get indices to rows matchings current depth slice (z)
+  # this is the default method
   if(include == 'top')
-    res <- d.v[which(z >= d.top & z < d.bottom)]
-  else
-    stop('invalid horizon boundary rule')
+    idx <- which(z >= h[[top]] & z < h[[bottom]])  
+  # not sure why someone would use this approach, but include it anyways  
+  if(include == 'bottom')
+  	idx <- which(z > h[[top]] & z <= h[[bottom]])
+
+  # 2. extract data.frame along slice, and named vars + id
+  h <- h[idx, c(id, vars)]
   
-  # used for QA/QC
-  l.res <- length(res)
+  # 3. QA/QC
+  # how many unique IDs?
+  l.ids <- length(unique(h[[id]]))   
+  # how many rows in the result?
+  n.res <- nrow(h)
   
-  # account for no data
-  if(l.res == 0)
-    res <- NA
+  # more rows than IDs --> bad horizonation
+  if(l.ids != n.res) {
+  	if(strict == TRUE) {
+  	  # get offending IDs
+  	  id.tab <- table(h[[id]])
+  	  bad.ids <- paste(names(id.tab)[which(id.tab > 1)], collapse=', ')
+  	  stop(paste('bad horizonation in IDs:', bad.ids))
+  	  }
+  	
+  	# looser interp of the data... issue warning and return multuple rows/ID
+  	# join(..., match='first') will correct the problem
+    else
+      warning('Bad horizonation detected, first matching horizon selected. Use strict=TRUE to enforce QA/QC.')
+  	}
   
-  # if there were multiple matches (i.e. bad horizonation)
-  if(l.res > 1 ) {
-    
-    # strict usage of the data... erors stop execution
-    if(strict == TRUE) {
-      print(d)
-      stop('bad horizonation')  
-    }
-    
-    # looser interp of the data... issue warning and pic the first
-    else {
-     warning('Bad horizonation detected, using the mean of all matching results. Use strict=TRUE to enforce QA/QC.')
-     res <- mean(res, na.rm=TRUE)
-    }
-    
-  }
-  
-  # name the variable, for nicer column names output from ddply()
-  names(res) <- 'slice'
-  return(res)
+  # done: return subset of original data
+  return(h)
   }
 
 
-## slice:
-if (!isGeneric("slice"))
-  setGeneric("slice", function(object, ...) standardGeneric("slice"))
-
-
-## TODO: this is slower than soil.slot ... why?
-## TODO: allow the use of site data (PSC etc.) to determine the z-slice
-setMethod(f='slice', signature='SoilProfileCollection',
-  function(object, fm, top.down=TRUE, just.the.data=FALSE, progress='none', strict=TRUE){
+## this is a much more robust + fast version of slice.slow
+## needs a little more testing, and then will be ready
+slice.fast <- function(object, fm, top.down=TRUE, just.the.data=FALSE, strict=TRUE){
   
   ## important: change the default behavior of data.frame and melt
   opt.original <- options(stringsAsFactors = FALSE)
@@ -85,46 +70,38 @@ setMethod(f='slice', signature='SoilProfileCollection',
   hd <- horizonDepths(object)
   top <- hd[1] ; bottom <- hd[2] # convenience vars
   id <- idname(object)
-  
-  # get variable classes
-  vars.is.numeric.test <- sapply(vars, function(i) is.numeric(h[[i]]))  
-  
-	# check for bogus left/right side problems with the formula
+  id.order <- profile_id(object) # this is the original ordering of profiles
+    
+  # check for bogus left/right side problems with the formula
   if(any(z < 0) | any(is.na(z)))
     stop('z-slice must be >= 1')
 
   ## TODO: this will have to be updated for z-slices defined by data in @site
-	if(! class(z) %in% c('numeric','integer')) # bogus z-slice
+  if(! class(z) %in% c('numeric','integer')) # bogus z-slice
 		stop('z-slice must be either numeric or integer')
 
-	if(any(vars %in% names(h)) == FALSE) # bogus column names in right-hand side
+  if(any(vars %in% names(h)) == FALSE) # bogus column names in right-hand side
 		stop('column names in formula do not match any horizon data')
 
-  # notify user that a mixture of numeric / categorical vars is not supported
-  if(length(unique(vars.is.numeric.test)) > 1) {
-     print(vars.is.numeric.test)
-     stop('a mixture of numeric/categoric variables is not currently supported')
-  }
   
-  ## TODO this approach won't work with mixed (numeric / char / factor variables)
-  # numeric / categorical vars must be done in different passes
-  # melt into long format
-  m <- melt(h, measure.vars=vars, id.vars=c(id, top, bottom))
-  
-  ## melt.data.frame will convert id column into a factor... undo that behavior!
-  m[[id]] <- as.character(m[[id]])
-  
-  # extract slice by id/variable, for each requested depth
+  ## extract all vars by slice_i
   # pre-allocate storage as list
   hd.slices <- vector(mode='list', length=length(z))
   # prepare an index for the list
   slice.idx <- seq_along(z)
-
+  
   # iterate over this index
   for(slice.i in slice.idx) {
     
-    # errors from get.slice() can be avoided loosening constraints with strict=FALSE
-    m.i <- ddply(m, c(id, 'variable'), .fun=get.slice, .progress=progress, top=top, bottom=bottom, z=z[slice.i], strict=strict)
+    # extract all vars for current slice
+    m.i.sub <- get.slice(h, id=id, top=top, bottom=bottom, vars=vars, z=z[slice.i], strict=strict)
+    
+    # join with original IDs in order to account for NA, or bad horizonation
+    d <- data.frame(temp_id=id.order)
+    names(d) <- id
+    
+    ## BUG: join doesn't work when ID is a factor
+    m.i <- join(d, m.i.sub, by=id, type='left', match='first')
     
     # add depth range:
     # top-down, means that the slice starts from the user-defined depths (default)
@@ -141,17 +118,13 @@ setMethod(f='slice', signature='SoilProfileCollection',
     hd.slices[[slice.i]] <- m.i
     }
   
-  # convert list into DF and order by id, top = hd[1]
+  # convert list into DF
   hd.slices <- ldply(hd.slices)
   
-  ## TODO: make sure sorting is correct!  
-  hd.slices <- hd.slices[order(hd.slices[[id]], hd.slices[[top]]), ]
-
-  # convert back into wide format
-  # and remove reshape-related attributes
-  fm.to.wide <- as.formula(paste(id, '+', top, '+', bottom, '~', 'variable', sep=' '))
-  hd.slices <- data.frame(cast(hd.slices, formula=fm.to.wide, value='slice'), stringsAsFactors=FALSE)
-
+  # re-order by id, then top
+  # keep only data we care about
+  hd.slices <- hd.slices[order(match(hd.slices[[id]], id.order), hd.slices[[top]]), c(id, top, bottom, vars)]
+  
   # if we just want the data:
   if(just.the.data)
     return(hd.slices)
@@ -162,7 +135,7 @@ setMethod(f='slice', signature='SoilProfileCollection',
     cat('result is a SpatialPointsDataFrame object\n')
     # check for site data, if present - join to our sliced data
     if(nrow(site(object)) > 0 )
-      hd.slices <- join(hd.slices, site(object))
+      hd.slices <- join(hd.slices, site(object), by=id)
     
     return(SpatialPointsDataFrame(coordinates(object), data=hd.slices))
     }
@@ -184,4 +157,13 @@ setMethod(f='slice', signature='SoilProfileCollection',
   # done
   return(hd.slices)
   }
-)
+
+
+## slice:
+if (!isGeneric("slice"))
+  setGeneric("slice", function(object, ...) standardGeneric("slice"))
+
+
+## TODO: this is slower than soil.slot ... why?
+## TODO: allow the use of site data (PSC etc.) to determine the z-slice
+setMethod(f='slice', signature='SoilProfileCollection', slice.fast)
