@@ -13,7 +13,7 @@ format_slices <- function(data, crs=NULL, depths, variable=NULL)
 	{
 		# get this depth slice
 		# 'top' must be a numeric column in the source dataframe
-		d.sub <- subset(data, subset=top==i)
+		d.sub <- d.sub[d.sub$top == i, ]
 		
 		# convert to SPDF: 'x' and 'y' columns must be present
 		coordinates(d.sub) <- ~ x+y	
@@ -39,3 +39,4 @@ format_slices <- function(data, crs=NULL, depths, variable=NULL)
 	}
 	return(slices)
 }
+
